@@ -5,16 +5,9 @@ commands on remote servers.
 A :class:`.Store` can also implement several methods for managing
 the higher level :ref:`object data mapper <odm>`.
 '''
-import logging
-import socket
-from functools import partial
-
-from pulsar import (get_event_loop, ImproperlyConfigured, Pool, new_event_loop,
-                    coroutine_return, get_application, in_loop,
-                    EventHandler, Producer)
+from pulsar import ImproperlyConfigured, Pool, Producer
 from pulsar.utils.importer import module_attribute
 from pulsar.utils.pep import to_string
-from pulsar.utils.system import json
 from pulsar.utils.httpurl import urlsplit, parse_qsl, urlunparse, urlencode
 
 __all__ = ['Command',
@@ -29,6 +22,9 @@ __all__ = ['Command',
 data_stores = {}
 
 
+REV_KEY = '_rev'
+
+
 class Command(object):
     '''A command executed during a in a :meth:`~.Store.execute_transaction`
 
@@ -61,10 +57,7 @@ class StoreTransaction(object):
         self.commands = []
 
     def add(self, model):
-        if '_rev' in model:
-            action = Command.UPDATE
-        else:
-            action = Command.INSERT
+        action = Command.UPDATE if REV_KEY in model else Command.INSERT
         self.commands.append(Command(model, action))
         return model
 
@@ -108,12 +101,11 @@ class Store(Producer):
     registered = False
     MANY_TIMES_EVENTS = ('request',)
 
-    def __init__(self, name, host, loop, database=None,
+    def __init__(self, name, host, loop=None, database=None,
                  user=None, password=None, encoding=None, **kw):
-        super(Producer, self).__init__()
+        super(Store, self).__init__(loop)
         self._name = name
         self._host = host
-        self._loop = loop
         self._encoding = encoding or 'utf-8'
         self._database = database
         self._user = user
@@ -309,16 +301,6 @@ class Store(Producer):
         instance['_store'] = self
         return instance
 
-    def model_data(self, model, action):
-        '''A generator of field/value pair for the store
-        '''
-        fields = model._meta.dfields
-        for key, value in model.items():
-            if key in fields:
-                value = fields[key].to_store(value, self)
-            if value is not None:
-                yield key, value
-
     def model_data(self, model, action):
         '''Generator of ``field, value`` pair for the data store.
 
@@ -516,7 +498,7 @@ def parse_store_url(url):
     return scheme, host, params
 
 
-def create_store(url, loop=None, **kw):
+def create_store(url, **kw):
     '''Create a new :class:`Store` for a valid ``url``.
 
     :param url: a valid ``url`` takes the following forms:
@@ -534,13 +516,6 @@ def create_store(url, loop=None, **kw):
             couchdb://user:password@127.0.0.1:6500/testdb
             https+couchdb://user:password@127.0.0.1:6500/testdb
 
-    :param loop: optional event loop, obtained by
-        :func:`~asyncio.get_event_loop` if not provided.
-        To create a synchronous client pass a new event loop created via
-        the :func:`~asyncio.new_event_loop`.
-        In the latter case the event loop is employed only for synchronous
-        type requests via the :meth:`~asyncio.BaseEventLoop.run_until_complete`
-        method.
     :param kw: additional key-valued parameters to pass to the :class:`.Store`
         initialisation method. It can contains parameters such as
         ``database``, ``user`` and ``password`` to override the
@@ -554,15 +529,12 @@ def create_store(url, loop=None, **kw):
     dotted_path = data_stores.get(scheme)
     if not dotted_path:
         raise ImproperlyConfigured('%s store not available' % scheme)
-    loop = loop or get_event_loop()
-    if not loop:
-        loop = new_event_loop(logger=logging.getLogger(dotted_path))
     store_class = module_attribute(dotted_path)
     if not store_class.registered:
         store_class.registered = True
         store_class.register()
     params.update(kw)
-    return store_class(scheme, address, loop, **params)
+    return store_class(scheme, address, **params)
 
 
 def register_store(name, dotted_path):
