@@ -1,8 +1,8 @@
 from functools import partial
 
-from pulsar import coroutine_return, task, Connection, Pool, get_actor
+from pulsar import coroutine_return, task, Connection, Pool, get_actor, From
 from pulsar.utils.pep import to_string
-from pulsar.apps.data import register_store, Store, Command
+from pulsar.apps.data import Store, Command
 from pulsar.apps.ds import redis_parser
 
 from .client import RedisClient, Pipeline, Consumer, ResponseError
@@ -95,15 +95,15 @@ class RedisStore(Store):
         protocol_factory = protocol_factory or self.create_protocol
         if isinstance(self._host, tuple):
             host, port = self._host
-            transport, connection = yield self._loop.create_connection(
-                protocol_factory, host, port)
+            transport, connection = yield From(self._loop.create_connection(
+                protocol_factory, host, port))
         else:
             raise NotImplementedError('Could not connect to %s' %
                                       str(self._host))
         if self._password:
-            yield connection.execute('AUTH', self._password)
+            yield From(connection.execute('AUTH', self._password))
         if self._database:
-            yield connection.execute('SELECT', self._database)
+            yield From(connection.execute('SELECT', self._database))
         coroutine_return(connection)
 
     def execute_transaction(self, transaction):
@@ -119,16 +119,13 @@ class RedisStore(Store):
                 pipe.execute(*command.args)
             elif action in update_insert:
                 model = command.args
+                model['_rev'] = model.get('_rev', 0) + 1
                 models.append(model)
                 key = self.basekey(model._meta, model.id)
                 pipe.hmset(key, self.model_data(model, action))
             else:
                 raise NotImplementedError
-        response = yield pipe.commit()
-        for command in transaction.commands:
-            if command.action == Command.INSERT:
-                model = command.args
-                model['_rev'] = 1
+        yield From(pipe.commit())
         coroutine_return(models)
 
     def get_model(self, manager, pk):
@@ -158,7 +155,7 @@ class RedisStore(Store):
 
     def meta(self, meta):
         '''Extract model metadata for lua script stdnet/lib/lua/odm.lua'''
-        indices = dict(((idx.attname, idx.unique) for idx in meta.indices))
+        #  indices = dict(((idx.attname, idx.unique) for idx in meta.indices))
         data = meta.as_dict()
         data['namespace'] = self.basekey(meta)
         return data
