@@ -37,7 +37,7 @@ PyRedisParser = lambda: Parser(InvalidResponse, response_error)
 if pulsar.HAS_C_EXTENSIONS:
     from pulsar.utils.lib import RedisParser as _RedisParser
     RedisParser = lambda: _RedisParser(InvalidResponse, response_error)
-else:
+else:    # pragma nocover
     RedisParser = PyRedisParser
 
 
