@@ -1,8 +1,6 @@
-import sys
-import unittest
-import logging
-
-from pulsar import multi_async, coroutine_return
+import pulsar
+from pulsar import (isfuture, run_in_loop, coroutine_return, From,
+                    multi_async, task)
 from pulsar.utils.pep import ispy3k
 from pulsar.apps import tasks
 
@@ -10,7 +8,7 @@ from .utils import TestFunction, TestFailure, is_expected_failure
 
 if ispy3k:
     from unittest import mock
-else:  # pragma nocover
+else:  # pragma     nocover
     try:
         import mock
     except ImportError:
@@ -29,7 +27,11 @@ class Test(tasks.Job):
         all_tests = runner.loadTestsFromTestCase(testcls)
         num = all_tests.countTestCases()
         if num:
-            return self.run(consumer, runner, testcls, all_tests)
+            if getattr(testcls, '_actor_execution', False):
+                return run_in_loop(consumer.worker._loop, self.run_testcls,
+                                   consumer, runner, testcls, all_tests)
+            else:
+                return self.run_testcls(consumer, runner, testcls, all_tests)
         else:
             return runner.result
 
@@ -38,7 +40,8 @@ class Test(tasks.Job):
         testcls = kwargs.get('testcls')
         return '%s_%s' % (testcls.__name__, tid) if testcls else tid
 
-    def run(self, consumer, runner, testcls, all_tests):
+    @task
+    def run_testcls(self, consumer, runner, testcls, all_tests):
         '''Run all test functions from the :attr:`testcls`.
 
         It uses the following algorithm:
@@ -50,7 +53,6 @@ class Test(tasks.Job):
           unless the test class should be skipped.
         '''
         cfg = testcls.cfg
-        loop = consumer._loop
         runner.startTestClass(testcls)
         error = None
         sequential = getattr(testcls, '_sequential_execution', cfg.sequential)
@@ -65,12 +67,13 @@ class Test(tasks.Job):
                 yield self.run_test(test, runner, error)
         else:
             all = (self.run_test(test, runner, error) for test in all_tests)
-            yield multi_async(all, loop=loop)
+            yield multi_async(all)
         if not skip_tests:
             yield self._run(runner, testcls, 'tearDownClass', add_err=False)
         runner.stopTestClass(testcls)
         coroutine_return(runner.result)
 
+    @task
     def run_test(self, test, runner, error=None):
         '''Run a ``test`` function using the following algorithm
 
@@ -124,10 +127,12 @@ class Test(tasks.Job):
                 method, '__unittest_expecting_failure__', False)
             if tfunc is None:
                 tfunc = TestFunction(method.__name__)
-            try:
-                exc = yield tfunc(test, test.cfg.test_timeout)
-            except Exception as e:
-                exc = e
+            exc = tfunc(test, test.cfg.test_timeout)
+            if isfuture(exc):
+                try:
+                    exc = yield From(exc)
+                except Exception as e:
+                    exc = e
             if exc:
                 add_err = False if previous else add_err
                 previous = self.add_failure(test, runner, exc, add_err,
