@@ -26,10 +26,6 @@ AsyncAssert
    :members:
    :member-order: bysource
 
-run test server
-~~~~~~~~~~~~~~~~~~~~~~
-
-.. autofunction:: run_test_server
 
 check server
 ~~~~~~~~~~~~~~~~~~
@@ -42,7 +38,6 @@ import logging
 import unittest
 from inspect import isclass
 from functools import partial
-from contextlib import contextmanager
 
 try:
     from unittest.case import _ExpectedFailure as ExpectedFailure
@@ -51,8 +46,9 @@ except ImportError:
 
 import pulsar
 from pulsar import (get_actor, send, multi_async, async, future_timeout,
-                    TcpServer, coroutine_return, new_event_loop,
-                    Future, format_traceback)
+                    TcpServer, coroutine_return, new_event_loop, task,
+                    format_traceback, ImproperlyConfigured, Future,
+                    yield_from, iscoroutine, chain_future)
 from pulsar.async.proxy import ActorProxyFuture
 from pulsar.utils.importer import module_attribute
 from pulsar.apps.data import create_store
@@ -65,14 +61,14 @@ __all__ = ['run_on_arbiter',
            'AsyncAssert',
            'show_leaks',
            'hide_leaks',
-           'run_test_server',
            'check_server',
-           'dont_run_with_thread']
+           'dont_run_with_thread',
+           'run_on_actor']
 
 
 LOGGER = logging.getLogger('pulsar.test')
 NOT_TEST_METHODS = ('setUp', 'tearDown', '_pre_setup', '_post_teardown',
-                    'setUpClass', 'tearDownClass', 'run_test_server')
+                    'setUpClass', 'tearDownClass')
 
 
 class TestFailure:
@@ -102,25 +98,38 @@ class TestCallable(object):
     __str__ = __repr__
 
     def __call__(self, actor):
-        result = async(self._call(actor))
-        return future_timeout(result, self.timeout) if self.timeout else result
-
-    def _call(self, actor):
-        # Coroutine to run an asynchronous test function
         __skip_traceback__ = True
         test = self.test
         if self.istest:
             test = actor.app.runner.before_test_function_run(test)
-        inject_async_assert(test)
+        inject_async_assert(self.test)
         test_function = getattr(test, self.method_name)
-        failure = None
         try:
-            yield test_function()
+            result = test_function()
         except Exception as exc:
-            failure = TestFailure(exc)
+            result = TestFailure(exc)
+        else:
+            # Wraps a generator with yield_form
+            timeout = self.timeout
+            if iscoroutine(result):
+                result = yield_from(result, timeout)
+                timeout = None
+            try:
+                result = future_timeout(async(result), timeout)
+                result.add_done_callback(partial(self._finish, actor))
+                return chain_future(result, callback=self._none)
+            except TypeError:
+                result = None
+        self._finish(actor)
+        return result
+
+    def _finish(self, actor, fut=None):
+        # Callback
         if self.istest:
             actor.app.runner.after_test_function_run(self.test)
-        coroutine_return(failure)
+
+    def _none(self, _):
+        pass
 
 
 class SafeTest(object):
@@ -139,6 +148,7 @@ class SafeTest(object):
         else:
             cls = self.test.__class__
             data = self.test.__dict__.copy()
+            data.pop('_loop', None)
         return ('%s.%s' % (cls.__module__, cls.__name__), data)
 
     def __setstate__(self, state):
@@ -189,6 +199,15 @@ def run_on_arbiter(f):
     return f
 
 
+def run_on_actor(cls):
+    '''Decorator for a :class:`~unittest.TestCase` which cause
+    its test functions to run on the actor evnet loop rather than
+    in the executor
+    '''
+    cls._actor_execution = True
+    return cls
+
+
 def sequential(cls):
     '''Decorator for a :class:`~unittest.TestCase` which cause
     its test functions to run sequentially rather than in an
@@ -230,20 +249,27 @@ class AsyncAssert(object):
 
     .. _descriptor: http://users.rcn.com/python/download/Descriptor.htm
     '''
-    def __init__(self, test=None):
+    def __init__(self, test):
         self.test = test
 
     def __get__(self, instance, instance_type=None):
-        return AsyncAssert(instance)
+        if instance is not None:
+            return AsyncAssert(instance)
+        else:
+            return self
 
     def __getattr__(self, name):
+
+        @task
         def _(*args, **kwargs):
             __skip_traceback__ = True
             args = yield multi_async(args)
             result = yield getattr(self.test, name)(*args, **kwargs)
             coroutine_return(result)
+
         return _
 
+    @task
     def assertRaises(self, error, callable, *args, **kwargs):
         try:
             yield callable(*args, **kwargs)
@@ -277,8 +303,10 @@ class ActorTestMixin(object):
             self._spawned = []
         return self._spawned
 
+    @task
     def spawn_actor(self, concurrency=None, **kwargs):
-        '''Spawn a new actor and perform some tests.'''
+        '''Spawn a new actor and perform some tests
+        '''
         concurrency = concurrency or self.concurrency
         ad = pulsar.spawn(concurrency=concurrency, **kwargs)
         self.assertTrue(ad.aid)
@@ -292,10 +320,7 @@ class ActorTestMixin(object):
 
     def stop_actors(self, *args):
         all = args or self.all_spawned
-        if len(all) == 1:
-            return send(all[0], 'stop')
-        elif all:
-            return multi_async((send(a, 'stop') for a in all))
+        return multi_async((send(a, 'stop') for a in all))
 
     def tearDown(self):
         return self.stop_actors()
@@ -304,7 +329,7 @@ class ActorTestMixin(object):
 def inject_async_assert(obj):
     tcls = obj if isclass(obj) else obj.__class__
     if not hasattr(tcls, 'async'):
-        tcls.async = AsyncAssert()
+        tcls.async = AsyncAssert(tcls)
 
 
 def show_leaks(actor, show=True):
@@ -332,24 +357,6 @@ def hide_leaks(actor):
     show_leaks(actor, False)
 
 
-@contextmanager
-def run_test_server(protocol_factory, loop, address=None, **kw):
-    '''A context manager for running a test server::
-
-        with run_test_server(loop, protocol_factory) as server:
-            ...
-
-    It creates a :class:`.TcpServer` and invoke
-    :meth:`~.TcpServer.stop_serving` on exit.
-    '''
-    address = address or ('127.0.0.1', 0)
-    server = TcpServer(protocol_factory, loop, address, **kw)
-    try:
-        yield server
-    finally:
-        server.stop_serving()
-
-
 def check_server(name):
     '''Check if server ``name`` is available at the address specified
     ``<name>_server`` config value.
@@ -357,12 +364,14 @@ def check_server(name):
     :rtype: boolean
     '''
     cfg = get_actor().cfg
-    cfgname = '%s_server' % name
     addr = cfg.get('%s_server' % name)
     if addr:
         if ('%s://' % name) not in addr:
             addr = '%s://%s' % (name, addr)
-        sync_store = create_store(addr, loop=new_event_loop())
+        try:
+            sync_store = create_store(addr, loop=new_event_loop())
+        except ImproperlyConfigured:
+            return False
         try:
             sync_store.ping()
             return True
