@@ -142,6 +142,7 @@ Html Factory
 .. _`HTML5 document`: http://www.w3schools.com/html/html5_intro.asp
 '''
 import os
+import re
 import json as pyjson
 
 from collections import Mapping, OrderedDict
@@ -153,39 +154,16 @@ from pulsar import multi_async, async, coroutine_return, chain_future
 from pulsar.utils.pep import iteritems, to_string, ispy3k
 from pulsar.utils.slugify import slugify
 from pulsar.utils.html import INLINE_TAGS, escape, dump_data_value, child_tag
-from pulsar.utils.httpurl import remove_double_slash
 from pulsar.utils.system import json
 
 from .html import html_visitor, newline
 
 __all__ = ['AsyncString', 'Html',
            'Json', 'HtmlDocument', 'Links', 'Scripts', 'Media',
-           'html_factory', 'media_libraries']
+           'html_factory']
 
 
-_libs_url = 'http://quantmind.github.io/jslibs/libs.json'
-_media_libraries = None
-
-
-def media_libraries():
-    global _media_libraries
-    if _media_libraries is None:
-        if os.path.isfile('libs.json'):     # pragma    nocover
-            with open('libs.json') as f:
-                data = f.read()
-            _media_libraries = json.loads(data)
-        else:
-            from pulsar import new_event_loop
-            from pulsar.apps.http import HttpClient
-            http = HttpClient(loop=new_event_loop())
-            try:
-                response = http.get(_libs_url)
-                _media_libraries = response.json()
-            except Exception:   # pragma    nocover
-                http.logger.error('Could not import media library',
-                                  exc_info=True)
-                _media_libraries = {'libs': {}, 'deps': {}}
-    return _media_libraries
+DATARE = re.compile('data[-_]')
 
 
 if ispy3k:
@@ -238,6 +216,7 @@ class AsyncString(object):
     _streamed = False
     _children = None
     _parent = None
+    _before_stream = None
     charset = None
 
     def __init__(self, *children, **params):
@@ -376,6 +355,9 @@ class AsyncString(object):
         if self._streamed:
             raise RuntimeError('%s already streamed' % self)
         self._streamed = True
+        if self._before_stream:
+            for cbk in self._before_stream:
+                cbk(request, self)
         return self.do_stream(request)
 
     def do_stream(self, request):
@@ -431,6 +413,16 @@ class AsyncString(object):
         '''
         return to_string(''.join(stream_to_string(streams)))
 
+    def before_render(self, callback):
+        '''Add a callback to be executed before this content is rendered
+
+        The callback accept ``request`` and ``self`` as the only
+        two arguments
+        '''
+        if not self._before_stream:
+            self._before_stream = []
+        self._before_stream.append(callback)
+
     def render(self, request=None):
         '''Render this string.
 
@@ -599,7 +591,7 @@ class Html(AsyncString):
 
     def _setup(self, cn=None, attr=None, css=None, data=None,
                content_type=None, **params):
-        self.charset = params.get('charset') or 'utf-8'
+        self.charset = params.pop('charset', None) or 'utf-8'
         self._content_type = content_type or self._default_content_type
         self._visitor = html_visitor(self._tag)
         self.addClass(cn)
@@ -616,9 +608,13 @@ class Html(AsyncString):
             return attr or {}
         result, adding = self._attrdata('attr', *args)
         if adding:
-            if attr is None:
-                self._extra['attr'] = attr = {}
-            attr.update(result)
+            for key, value in result.items():
+                if DATARE.match(key):
+                    self.data(key[5:], value)
+                else:
+                    if attr is None:
+                        self._extra['attr'] = attr = {}
+                    attr[key] = value
             result = self
         return result
 
@@ -788,30 +784,16 @@ class Media(AsyncString):
         rispectively.
 
         Default: ``False``
-
-    .. attribute:: known_libraries
-
-        Optional dictionary of known media libraries, mapping a name to a
-        valid absolute or local url. For example::
-
-            known_libraries = {'jquery':
-                               '//code.jquery.com/jquery-1.9.1.min.js'}
-
-        Default: ``None``
     '''
     mediatype = None
 
-    def __init__(self, media_path, minified=False, known_libraries=None,
-                 dependencies=None):
+    def __init__(self, media_path, minified=False, asset_protocol=None):
         super(Media, self).__init__()
         self.media_path = media_path
+        self.asset_protocol = asset_protocol
+        if self.media_path and not self.media_path.endswith('/'):
+            self.media_path = '%s/' % self.media_path
         self.minified = minified
-        if known_libraries is None:
-            known_libraries = media_libraries()['libs']
-        if dependencies is None:
-            dependencies = media_libraries()['deps']
-        self.known_libraries = known_libraries
-        self.dependencies = dependencies
 
     def is_relative(self, path):
         '''Check if ``path`` is a local relative path.
@@ -822,50 +804,32 @@ class Media(AsyncString):
         return not (path.startswith('http://') or path.startswith('https://')
                     or path.startswith('/'))
 
-    def absolute_path(self, path, with_media_ending=True):
+    def absolute_path(self, path):
         '''Return a suitable absolute url for ``path``.
 
-        The url is calculated in the following way:
-
-        * Check if ``path`` is an entry in the :attr:`known_libraries`
-          dictionary. In this case replace ``path`` with
-          ``known_libraries[path]``.
-        * If ``path`` :meth:`is_relative` build a sutable url by prepending
-          the :attr:`media_path` attribute.
+        If ``path`` :meth:`is_relative` build a sutable url by prepending
+        the :attr:`media_path` attribute.
 
         :return: A url path to insert in a HTML ``link`` or ``script``.
         '''
         ending = '.%s' % self.mediatype
-        minify = True
-        urlparams = ''
-        if isinstance(path, dict):
-            urlparams = path.get('urlparams', urlparams)
-            minify = path.get('minify', minify)
-            path = path['url']
-        if path in self.known_libraries:
-            lib = self.known_libraries[path]
-            if isinstance(lib, dict):
-                urlparams = lib.get('urlparams', '')
-                minify = lib.get('minify', minify)
-                lib = lib['url']
-            path = '%s%s' % (lib, ending)
-        if self.minified and minify:
-            if path.endswith(ending):
-                path = self._minify(path, ending)
-        if not with_media_ending:
-            path = path[:-len(ending)]
-        if urlparams:
-            path = '%s?%s' % (path, urlparams)
-        if self.is_relative(path):
-            return remove_double_slash('%s/%s' % (self.media_path, path))
+        if not path.endswith(ending):
+            if self.minified:
+                path = '%s.min' % path
+            path = '%s%s' % (path, ending)
+        #
+        if self.is_relative(path) and self.media_path:
+            return '%s%s' % (self.media_path, path)
+        elif self.asset_protocol and path.startswith('//'):
+            return '%s%s' % (self.asset_protocol, path)
         else:
             return path
 
-    def _minify(self, path, postfix):
-        new_postfix = 'min%s' % postfix
-        if not path.endswith(new_postfix):
-            path = '%s.%s' % (path[:-len(postfix)], new_postfix)
-        return path
+    def extend(self, iterable):
+        '''Add a list (iterable) of media to this container
+        '''
+        for media in iterable:
+            self.append(media)
 
 
 class Links(Media):
@@ -886,16 +850,20 @@ class Links(Media):
         :param rel: Specifies the relationship between the document
             and the linked document. If not given ``stylesheet`` is used.
         :param type: Specifies the content type of the linked document.
-            If not given ``text/css`` is used.
+            If not given ``text/css`` is used. It an empty string is given,
+            it won't be added.
         :param media: Specifies on what device the linked document will be
             displayed. If not given or ``all``, the media is for all devices.
         :param kwargs: additional attributes
         '''
         if href:
-            type = type or 'text/css'
-            rel = rel or 'stylesheet'
             path = self.absolute_path(href)
-            value = Html('link', href=path, type=type, rel=rel, **kwargs)
+            if path.endswith('.css'):
+                type = type or 'text/css'
+                rel = rel or 'stylesheet'
+            value = Html('link', href=path, rel=rel, **kwargs)
+            if type:
+                value.attr('type', type)
             if media not in (None, 'all'):
                 value.attr('media', media)
             if condition:
@@ -905,6 +873,8 @@ class Links(Media):
             if value not in self.children:
                 self.children.append(value)
 
+requires = ['require', 'requirejs']
+
 
 class Scripts(Media):
     '''A :class:`.Media` container for ``script`` tags.
@@ -914,10 +884,9 @@ class Scripts(Media):
     mediatype = 'js'
 
     def __init__(self, *args, **kwargs):
-        self.require_callback = kwargs.pop('require_callback', None)
         self.wait = kwargs.pop('wait', 200)
-        self.required = []
-        self._requirejs = False
+        self.require = []
+        self.paths = {}
         super(Scripts, self).__init__(*args, **kwargs)
 
     def script(self, src, type=None, **kwargs):
@@ -925,28 +894,6 @@ class Scripts(Media):
         path = self.absolute_path(src)
         return Html('script', src=path, type=type, **kwargs).render()
 
-    def require(self, *scripts):
-        '''Add a ``script`` to the list of :attr:`required` scripts.
-
-        The ``script`` can be a name in the :attr:`~Media.known_libraries`,
-        an absolute uri or a relative url.
-
-        The script will be loaded using the ``require`` javascript package.
-        '''
-        if scripts:
-            self.append('require')
-        required = self.required
-        for script in scripts:
-            if script == 'require':
-                continue
-            name = script
-            if isinstance(script, dict):
-                name = script['url']
-            if name.strip() not in self.known_libraries:
-                script = self.absolute_path(script)
-            if script not in required:
-                required.append(script)
-
     def append(self, src=None, type=None, **kwargs):
         '''add a new script to the container.
 
@@ -955,33 +902,23 @@ class Scripts(Media):
             case the :attr:`Media.media_path` attribute is prepended.
         '''
         if src:
-            if src == 'require':
-                self._requirejs = True
             script = self.script(src, type=type, **kwargs)
             if script not in self.children:
                 self.children.append(script)
 
     def require_script(self):
         '''Can be used for requirejs'''
-        libs = dict(((key, self.absolute_path(key, False))
-                     for key in self.known_libraries))
-        return OrderedDict((('deps', self.required),
-                            ('paths', libs),
-                            ('shim', self.dependencies),
-                            ('waitSeconds', self.wait)))
+        return {'deps': [r for r in self.require if r not in requires],
+                'paths': self.paths,
+                'baseUrl': self.media_path,
+                'minify': self.minified,
+                'waitSeconds': self.wait}
 
     def do_stream(self, request):
-        if self._requirejs or self.required:
-            require = self.require_script()
-            callback = self.require_callback or ''
-            if callback:
-                callback = ('\nrequire.callback = function () {%s();}'
-                            % callback)
+        if self.require:
             yield ('<script type="text/javascript">\n'
-                   'var require = %s,\n'
-                   '    media_path = "%s";%s\n'
-                   '</script>\n') % (pyjson.dumps(require),
-                                     self.media_path, callback)
+                   'var require = %s;\n'
+                   '</script>\n') % json.dumps(self.require_script())
         for child in self.children:
             yield child
 
@@ -1055,20 +992,16 @@ class Head(Html):
 
     '''
     def __init__(self, media_path=None, title=None, meta=None, minified=False,
-                 known_libraries=None, scripts_dependencies=None,
-                 require_callback=None, **params):
+                 asset_protocol=None, **params):
         super(Head, self).__init__('head', **params)
         self.title = title
         self.append(Html(None, meta))
         self.append(Links(media_path, minified=minified,
-                          known_libraries=known_libraries,
-                          dependencies=scripts_dependencies))
+                          asset_protocol=asset_protocol))
         self.append(Embedded('style', type='text/css'))
         self.append(Embedded('script', type='text/javascript'))
         self.append(Scripts(media_path, minified=minified,
-                            known_libraries=known_libraries,
-                            dependencies=scripts_dependencies,
-                            require_callback=require_callback))
+                            asset_protocol=asset_protocol))
         self.add_meta(charset=self.charset)
 
     @property
@@ -1118,11 +1051,10 @@ class Head(Html):
 
     def add_meta(self, **kwargs):
         '''Add a new :class:`Html` meta tag to the :attr:`meta` collection.'''
-        meta = Html('meta', **kwargs)
-        self.meta.append(meta)
+        self.meta.append(Html('meta').attr(kwargs))
 
-    def get_meta(self, name):
-        '''Get the ``content`` attribute of meta tag ``name``.
+    def get_meta(self, name, meta_key=None):
+        '''Get the ``content`` attribute of a meta tag ``name``.
 
         For example::
 
@@ -1130,22 +1062,36 @@ class Head(Html):
 
         returns the ``content`` attribute of the meta tag with attribute
         ``name`` equal to ``description`` or ``None``.
+        If a different meta key needs to be matched, it can be specified via
+        the ``meta_key`` parameter::
+
+            head.get_meta('og:title', meta_key='property')
         '''
+        meta_key = meta_key or 'name'
         for child in self.meta._children:
-            if child.attr('name') == name:
+            if isinstance(child, Html) and child.attr(meta_key) == name:
                 return child.attr('content')
 
-    def replace_meta(self, name, content):
+    def replace_meta(self, name, content=None, meta_key=None):
         '''Replace the ``content`` attribute of meta tag ``name``
 
         If the meta with ``name`` is not available, it is added, otherwise
-        its content is replaced.
+        its content is replaced. If ``content`` is not given or it is empty
+        the meta tag with ``name`` is removed.
         '''
-        for child in self.meta._children:
-            if child.attr('name') == name:
-                child.attr('content', content)
+        children = self.meta._children
+        if not content:     # small optimazation
+            children = tuple(children)
+        meta_key = meta_key or 'name'
+        for child in children:
+            if child.attr(meta_key) == name:
+                if content:
+                    child.attr('content', content)
+                else:
+                    self.meta._children.remove(child)
                 return
-        self.add_meta(name=name, content=content)
+        if content:
+            self.add_meta(**{meta_key: name, 'content': content})
 
     def __add__(self, other):
         if isinstance(other, Media):
@@ -1176,20 +1122,14 @@ class HtmlDocument(Html):
                  '</html>')
 
     def __init__(self, title=None, media_path='/media/', charset=None,
-                 minified=False, known_libraries=None, require_callback=None,
-                 scripts_dependencies=None, **params):
+                 minified=False, loop=None, asset_protocol=None, **params):
         super(HtmlDocument, self).__init__(None, **params)
         self.head = Head(title=title, media_path=media_path, minified=minified,
-                         known_libraries=known_libraries,
-                         require_callback=require_callback,
-                         scripts_dependencies=scripts_dependencies,
-                         charset=charset)
+                         charset=charset, asset_protocol=asset_protocol)
         self.body = Html('body')
-        self.end = Html(None)
 
     def do_stream(self, request):
         # stream the body
-        self.body.append(self.end)
         body = multi_async(self.body.stream(request))
         # the body has asynchronous components
         # delay the header untl later
