@@ -45,7 +45,7 @@ class FakeConnection(object):
         self._loop = loop
 
 
-def test_wsgi_environ(path='/', method=None, headers=None, extra=None,
+def test_wsgi_environ(path=None, method=None, headers=None, extra=None,
                       secure=False, loop=None):
     '''An function to create a WSGI environment dictionary for testing.
 
@@ -58,15 +58,19 @@ def test_wsgi_environ(path='/', method=None, headers=None, extra=None,
     '''
     parser = http_parser(kind=0)
     method = (method or 'GET').upper()
-    path = iri_to_uri(path)
-    data = '%s %s HTTP/1.1\r\n\r\n' % (method, path)
-    data = data.encode('latin1')
-    parser.execute(data, len(data))
+    path = iri_to_uri(path or '/')
     request_headers = Headers(headers, kind='client')
     # Add Host if not available
     parsed = urlparse(path)
-    if parsed.netloc and 'host' not in request_headers:
-        request_headers['host'] = parsed.netloc
+    if 'host' not in request_headers:
+        if not parsed.netloc:
+            path = '%s%s' % ('https://:443' if secure else 'http://:80', path)
+        else:
+            request_headers['host'] = parsed.netloc
+    #
+    data = '%s %s HTTP/1.1\r\n\r\n' % (method, path)
+    data = data.encode('latin1')
+    parser.execute(data, len(data))
     #
     headers = Headers()
     stream = StreamReader(request_headers, parser)
@@ -299,6 +303,15 @@ class HttpServerResponse(ProtocolConsumer):
         self.keep_alive = False
         self.SERVER_SOFTWARE = server_software or self.SERVER_SOFTWARE
 
+    @property
+    def headers_sent(self):
+        '''Available once the headers have been sent to the client.
+
+        These are the bytes representing the first response line and
+        the headers
+        '''
+        return self._headers_sent
+
     def data_received(self, data):
         '''Implements :meth:`~.ProtocolConsumer.data_received` method.
 
