@@ -1,8 +1,8 @@
 import logging
 import threading
 import weakref
-from multiprocessing import dummy, current_process
 from functools import partial
+from multiprocessing import dummy, current_process
 
 try:
     import queue
@@ -12,22 +12,18 @@ ThreadQueue = queue.Queue
 Empty = queue.Empty
 Full = queue.Full
 
-from .access import (asyncio, selectors, new_event_loop, get_actor, set_actor,
-                     events, thread_data, _StopError, BaseEventLoop)
+from .access import (asyncio, selectors, get_actor, set_actor,
+                     events, thread_data, _StopError, BaseEventLoop,
+                     get_event_loop, logger as get_logger)
 from .futures import Future, Task, async, AsyncObject
 from .consts import ACTOR_STATES
 
 
-__all__ = ['Thread', 'IOqueue', 'ThreadPool', 'ThreadQueue', 'Empty', 'Full']
+__all__ = ['Thread', 'IOqueue', 'ThreadPool',
+           'ThreadQueue', 'Empty', 'Full']
 
 _MAX_WORKERS = 50
 _threads_queues = weakref.WeakKeyDictionary()
-passthrough = lambda: None
-
-
-def set_as_loop(loop):
-    if loop._iothreadloop:
-        asyncio.set_event_loop(loop)
 
 
 def get_executor(loop):
@@ -53,19 +49,21 @@ def run_in_executor(loop, executor, callback, *args):
 
 
 class Thread(dummy.DummyProcess):
+    _loop = None
+    _pool_loop = None
+
     @property
     def pid(self):
         return current_process().pid
 
-    def loop(self):
-        raise NotImplemented
-
     def terminate(self):
         '''Invoke the stop on the event loop method.'''
-        if self.is_alive():
-            loop = self.loop()
-            if loop:
-                loop.stop()
+        if self.is_alive() and self._loop:
+            self._loop.call_soon_threadsafe(self._loop.stop)
+
+    def set_loop(self, loop):
+        assert self._loop is None
+        self._loop = loop
 
 
 class PoolThread(Thread):
@@ -73,6 +71,7 @@ class PoolThread(Thread):
     '''
     def __init__(self, pool):
         self.pool = pool
+        self._pool_loop = pool._loop
         super(PoolThread, self).__init__(name=pool.worker_name)
 
     def __repr__(self):
@@ -87,21 +86,18 @@ class PoolThread(Thread):
         '''
         if self.pool._actor:
             set_actor(self.pool._actor)
-        asyncio.set_event_loop(self.pool._loop)
         # The run method for the threads in this thread pool
         logger = logging.getLogger('pulsar.%s' % self.name)
         loop = QueueEventLoop(self.pool, logger=logger, iothreadloop=True)
+        self.set_loop(loop)
         loop.run_forever()
 
-    def loop(self):
-        return thread_data('_request_loop', ct=self)
-
 
 class IOqueue(selectors.BaseSelector):
     '''A selector based on a distributed queue
 
     Since there is no way to my knowledge to wake up the queue while
-    getting an itiem from the task queue, the timeout cannot be larger than
+    getting an item from the task queue, the timeout cannot be larger than
     a small number which by default is ``0.5`` seconds.
     '''
     max_timeout = 0.5
@@ -141,22 +137,34 @@ class IOqueue(selectors.BaseSelector):
             raise _StopError
         return task
 
-    def process_task(self, task):
+    def process_task(self, task, loop):
         self._received += 1
-        future, func, args, kwargs = task
+        p, func, args, kwargs = task
         try:
-            result = yield func(*args, **kwargs)
+            result = func(*args, **kwargs)
         except Exception as exc:
-            self._completed += 1
-            future.set_exception(exc)
+            self._done_task(p, None, exc)
         else:
-            self._completed += 1
             try:
-                future.set_result(result)
-            finally:
-                # IMPORTANT: make sure to wake up the loop of the
-                # waiting future
-                future._loop.call_soon_threadsafe(passthrough)
+                result = async(result, loop=loop)
+            except TypeError:
+                self._done_task(p, None, result)
+            else:
+                result.add_done_callback(partial(self._done_task, p))
+
+    def _done_task(self, p, future, result=None):
+        self._completed += 1
+        #
+        if future:
+            try:
+                result = future.result()
+            except Exception as exc:
+                result = exc
+        #
+        if isinstance(result, Exception):
+            p._loop.call_soon_threadsafe(lambda: p.set_exception(result))
+        else:
+            p._loop.call_soon_threadsafe(lambda: p.set_result(result))
 
     def get_map(self):
         return {}
@@ -168,23 +176,42 @@ class IOqueue(selectors.BaseSelector):
         pass
 
 
-class QueueEventLoop(BaseEventLoop):
-    task_factory = Task
+class ThreadSafeLoop(object):
 
+    def __init__(self, iothreadloop):
+        self._iothreadloop = iothreadloop
+        if self._iothreadloop:
+            self._original_call_soon = self.call_soon
+            self.call_soon = self._threadsafe_call_soon
+            asyncio.set_event_loop(self)
+
+    def _threadsafe_call_soon(self, callback, *args):
+        if self != get_event_loop():
+            return self.call_soon_threadsafe(callback, *args)
+        else:
+            return self._original_call_soon(callback, *args)
+
+
+class QueueEventLoop(BaseEventLoop, ThreadSafeLoop):
+    '''An :ref:`asyncio event loop <asyncio-event-loop>` which
+    uses :class:`.IOqueue` as its selector.
+    '''
     def __init__(self, executor, iothreadloop=False, logger=None):
         super(QueueEventLoop, self).__init__()
+        ThreadSafeLoop.__init__(self, iothreadloop)
         self._default_executor = executor
-        self._iothreadloop = iothreadloop
         self._selector = IOqueue(executor)
-        self.logger = logger or LOGGER
-        self.call_soon(set_as_loop, self)
+        self.logger = get_logger(logger=logger)
+
+    def create_task(self, coro):
+        return Task(coro, loop=self)
 
     def _write_to_self(self):
         pass
 
     def _process_events(self, task):
         if task:
-            async(self._selector.process_task(task), self)
+            self._selector.process_task(task, self)
 
     def run_in_executor(self, executor, callback, *args):
         return run_in_executor(self, executor, callback, *args)
@@ -206,7 +233,7 @@ class ThreadPool(AsyncObject):
             if not max_workers:
                 max_workers = actor.cfg.thread_workers
             self.worker_name = '%s.%s' % (actor.name, self.worker_name)
-        self._loop = loop or asyncio.get_event_loop()
+        self._loop = loop or get_event_loop()
         self._max_workers = min(max_workers or _MAX_WORKERS, _MAX_WORKERS)
         self._threads = set()
         self._maxtasks = maxtasks
