@@ -11,7 +11,7 @@ from .base import *
 
 if platform.type == 'win':    # pragma nocover
     from .windowssystem import *
-elif platform.is_appengine:
+elif platform.is_appengine:    # pragma nocover
     from .appengine import *
 else:
     from .posixsystem import *
